﻿import QtQuick 2.0

import "Config.js" as Config
import "global.js" as Const

import XmlLoader 1.0


Item
{
    id: highwayLoadItem
    width: Config.HighwayLoadWidgetWidth
    height: Config.HighwayLoadWidgetHeight

    property DataLoader dataLoaderObj: DataLoader {
        proxyHostName : Config.ProxyHostName
        proxyHostPort : Config.ProxyHostPort

        dataUrl: Config.DataUrl
        updateTime : Config.DataUpdateTime
        makeSubstitution : Config.MakeSubstitution
        substitutionUrl: Config.SubstitutionUrl
        sortHighwaysByLevel: Config.SortHighwaysByLevel
        highwaysToIgnore: Config.HighwaysToIgnore
        onDataChanged: { if( !drumTimer.running ) drumTimer.start() }
    }

    HighwayLoadItemBase
    {
        id: bannerItem
        state: Const.stateActive
        animationDuration : Config.HighwayLoadAnimationDuration
        scrollDirection : Config.HighwayLoadScrollDirection

        Text
        {
            FontLoader
            {
                id: bannerFont
                source: Config.HighwayBannerTextFontSource
            }

            Image
            {
                id: bannerImage
                visible: Config.HighwayBannerImageVisible
                height: Config.HighwayBannerImageHeight
                width: Config.HighwayBannerImageWidth
                x: Config.HighwayBannerImageXPos
                y: Config.HighwayBannerImageYPos
                source: Config.HighwayBannerSourceFile
            }

            id: bannerText
            x: Config.HighwayBannerTextPositionX
            y: Config.HighwayBannerTextPositionY
            font.pointSize: Config.HighwayBannerTextFontSize
            font.family: bannerFont.name
            font.bold: Config.HighwayBannerTextFontBold
            verticalAlignment: Text.AlignVCenter
            color: Config.HighwayLoadLevelItemTextColorDefault
            text: Config.HighwayBannerImageVisible ? "По данным сервиса" : "По данным сервиса Яндекс.Пробки"
        }
    }

    property list<HighwayLoadItemName> loadItemName: [
        HighwayLoadItemName
        {
            parent: highwayLoadItem
            state: Config.HighwayLoadScrollDirection ? Const.stateInactive : Const.stateDisactive
            animationDuration : Config.HighwayLoadAnimationDuration
            scrollDirection : Config.HighwayLoadScrollDirection

        },

        HighwayLoadItemName
        {
            parent: highwayLoadItem
            state: Config.HighwayLoadScrollDirection ? Const.stateInactive : Const.stateDisactive
            animationDuration : Config.HighwayLoadAnimationDuration
            scrollDirection : Config.HighwayLoadScrollDirection

        },

        HighwayLoadItemName
        {
            parent: highwayLoadItem
            state: Config.HighwayLoadScrollDirection ? Const.stateDisactive : Const.stateInactive
            animationDuration : Config.HighwayLoadAnimationDuration
            scrollDirection : Config.HighwayLoadScrollDirection

        }
    ]

    property list<HighwayLoadItemDirection> loadItemDirection: [
        HighwayLoadItemDirection
        {
            parent: highwayLoadItem
            state: Config.HighwayLoadScrollDirection ? Const.stateInactive : Const.stateDisactive
            animationDuration : Config.HighwayLoadAnimationDuration
            scrollDirection : Config.HighwayLoadScrollDirection

        },

        HighwayLoadItemDirection
        {
            parent: highwayLoadItem
            state: Config.HighwayLoadScrollDirection ? Const.stateInactive : Const.stateDisactive
            animationDuration : Config.HighwayLoadAnimationDuration
            scrollDirection : Config.HighwayLoadScrollDirection

        },

        HighwayLoadItemDirection
        {
            parent: highwayLoadItem
            state: Config.HighwayLoadScrollDirection ? Const.stateDisactive : Const.stateInactive
            animationDuration : Config.HighwayLoadAnimationDuration
            scrollDirection : Config.HighwayLoadScrollDirection

        }
    ]

    Timer
    {
        id: drumTimer
        interval: Config.HighwayLoadScrollingInterval
        repeat: true
        //        running: true

        property int currentModelDirectionIndex: 0
        property int currentModelNameIndex: 0
        property int lastIndexName : 0
        property int lastIndexDirection : 0
        onTriggered: {
            if( dataLoaderObj.streetsCount === 0 || dataLoaderObj.directionsCount === 0 )
                return;

            var i =0;

            // first item
            if( bannerItem.state === Const.stateActive )
            {
                loadItemName[lastIndexName].highwayName = dataLoaderObj.getStreetTitle(currentModelNameIndex);
                loadItemName[lastIndexName].highwayLevel = dataLoaderObj.getDirectionLevel(currentModelNameIndex, currentModelDirectionIndex);

                loadItemDirection[lastIndexDirection].highwayDirection = dataLoaderObj.getDirectionName(currentModelNameIndex, currentModelDirectionIndex);
                loadItemDirection[lastIndexDirection].highwayLevel = dataLoaderObj.getDirectionLevel(currentModelNameIndex, currentModelDirectionIndex);

                loadItemDirection[lastIndexDirection].posX = Config.HighwayLoadLevelItemX
                loadItemName[lastIndexName].textWidth = Config.HighwayNameTextWidth

                if( Config.HighwayLoadDynamicText )
                {
                    if( Math.round(loadItemDirection[lastIndexDirection].paintedWidth) + 20 < Config.HighwayLoadWidgetWidth - Config.HighwayLoadLevelItemX )
                    {
                        loadItemName[lastIndexName].textWidth = Config.HighwayLoadWidgetWidth - Math.round(loadItemDirection[lastIndexDirection].paintedWidth) - 50;
                        if( loadItemName[lastIndexName].paintedWidth + 30 < loadItemName[lastIndexName].textWidth  )
                            loadItemDirection[lastIndexDirection].posX = Math.round(loadItemName[lastIndexName].paintedWidth) + 30
                        else
                            loadItemDirection[lastIndexDirection].posX = Config.HighwayLoadWidgetWidth - Math.round(loadItemDirection[lastIndexDirection].paintedWidth) - 20;
                    }
                    else
                    {
                        loadItemName[lastIndexName].textWidth = Config.HighwayNameTextWidth
                        if( Math.round(loadItemName[lastIndexName].paintedWidth) < loadItemName[lastIndexName].textWidth )
                            loadItemDirection[lastIndexDirection].posX = Math.round(loadItemName[lastIndexName].paintedWidth) + 35;
                    }
                }

                loadItemName[lastIndexName].getNextState();
                loadItemDirection[lastIndexDirection].getNextState();

                bannerItem.getNextState();
                return;
            }

            if ( ( Config.HighwayLoadScrollDirection && bannerItem.state === Const.stateDisactive ) ||
                    ( !Config.HighwayLoadScrollDirection && bannerItem.state === Const.stateInactive ) )
            {
                bannerItem.getNextState();
            }

            if( dataLoaderObj.getStreetCountDirections(currentModelNameIndex) === 2 )
            {
                if( currentModelDirectionIndex === 0 )
                {
                    currentModelDirectionIndex++;
                }
                else
                {
                    currentModelNameIndex++;
                    currentModelDirectionIndex = 0;
                }
            }
            else
            {
                currentModelNameIndex++;
                currentModelDirectionIndex = 0;
            }

            // last item
            if( currentModelNameIndex === dataLoaderObj.streetsCount )
            {
                bannerItem.getNextState();
                for ( i=0; i<3; i++ )
                {
                    if( Config.HighwayLoadScrollDirection )
                    {
                        if ( loadItemName[i].state === Const.stateInactive )
                            lastIndexName = i;
                        else
                            loadItemName[i].getNextState();

                        if( loadItemDirection[i].state === Const.stateInactive )
                            lastIndexDirection = i;
                        else
                            loadItemDirection[i].getNextState();

                    }
                    else
                    {
                        if ( loadItemName[i].state === Const.stateDisactive )
                            lastIndexName = i;
                        else
                            loadItemName[i].getNextState();

                        if( loadItemDirection[i].state === Const.stateDisactive )
                            lastIndexDirection = i;
                        else
                            loadItemDirection[i].getNextState();
                    }
                }
                currentModelDirectionIndex = 0;
                currentModelNameIndex = 0;
                return;
            }

            if( currentModelDirectionIndex === 0 )
                lastIndexName = ( lastIndexName + 1 )% 3
            lastIndexDirection = ( lastIndexDirection + 1 ) % 3

            loadItemName[lastIndexName].highwayName = dataLoaderObj.getStreetTitle(currentModelNameIndex);
            loadItemDirection[lastIndexDirection].highwayDirection = dataLoaderObj.getDirectionName(currentModelNameIndex, currentModelDirectionIndex);

            loadItemName[lastIndexName].highwayLevel = dataLoaderObj.getDirectionLevel(currentModelNameIndex, currentModelDirectionIndex);
            loadItemDirection[lastIndexDirection].highwayLevel = dataLoaderObj.getDirectionLevel(currentModelNameIndex, currentModelDirectionIndex);

            loadItemDirection[lastIndexDirection].posX = Config.HighwayLoadLevelItemX
            loadItemName[lastIndexName].textWidth = Config.HighwayNameTextWidth

            if( Config.HighwayLoadDynamicText )
            {
                if( Math.round(loadItemDirection[lastIndexDirection].paintedWidth) + 20 < Config.HighwayLoadWidgetWidth - Config.HighwayLoadLevelItemX )
                {
                    loadItemName[lastIndexName].textWidth = Config.HighwayLoadWidgetWidth - Math.round(loadItemDirection[lastIndexDirection].paintedWidth) - 50;
                    if( loadItemName[lastIndexName].paintedWidth + 30 < loadItemName[lastIndexName].textWidth  )
                        loadItemDirection[lastIndexDirection].posX = Math.round(loadItemName[lastIndexName].paintedWidth) + 30
                    else
                        loadItemDirection[lastIndexDirection].posX = Config.HighwayLoadWidgetWidth - Math.round(loadItemDirection[lastIndexDirection].paintedWidth) - 20;
                }
                else
                {
                    loadItemName[lastIndexName].textWidth = Config.HighwayNameTextWidth
                    if( Math.round(loadItemName[lastIndexName].paintedWidth) < loadItemName[lastIndexName].textWidth )
                        loadItemDirection[lastIndexDirection].posX = Math.round(loadItemName[lastIndexName].paintedWidth) + 35;
                }
            }

            console.log( Math.round(loadItemDirection[lastIndexDirection].posX) );

            for( i=0;i<loadItemName.length;i++ )
            {
                loadItemDirection[i].getNextState();
                if( currentModelDirectionIndex === 0 )
                    loadItemName[i].getNextState();
            }

            delete i;
            gc();

        } // onTriggered


    } // timer
} // item
